#include "Preferences_Constants.h"
#include "IMImage.h"
#include "ImageRoutinen.h"

// #include "magick/attribute.h"

IMImage::IMImage(void)
{
	image = NULL;
	LastError[0] = 0;
	Initialized = false;
}

IMImage::~IMImage(void)
{
	// no need to deallocate image
}

int IMImage::read(char * path)
{
	try { 
		image.read( path );  
		Initialized = true;
		return 0;
		}
	catch( Exception &error_ ) 
	{
		strlcpy(LastError, error_.what(), 255);
		return ImageMagickErrorFramework;
	}
}

int IMImage::read(void * data, const int length)
{
	try { 
		Blob myblob(data, length);
		image.read( myblob );
		Initialized = true;
		return 0;
		}
	catch( Exception &error_ ) 
	{
		strlcpy(LastError, error_.what(), 255);
		return ImageMagickErrorFramework;
	}
}

int IMImage::create(int x, int y, char * color)
{
	try { 
		Image newimage(Geometry(x, y), Color(color));
		image = newimage;
		Initialized = true;
		return 0;
		}
	catch( Exception &error_ ) 
	{
		strlcpy(LastError, error_.what(), 255);
		return ImageMagickErrorFramework;
	}
}

int IMImage::write(char * path)
{
	if (!Initialized)
		return ImageMagickErrorUnitialized;

	if (!image.isValid())
		return ImageMagickErrorUnitialized;

	try { 
		image.write( path );
		return 0;
		}
	catch( Exception &error_ ) 
	{
		strlcpy(LastError, error_.what(), 255);
		return ImageMagickErrorFramework;
	}
}


int IMImage::write(PA_Handle data)
{
	if (!Initialized)
		return ImageMagickErrorUnitialized;
	
	if (!image.isValid())
		return ImageMagickErrorUnitialized;

	try { 
		Blob myblob;
		image.write( &myblob );

		PA_SetHandleSize(data, myblob.length());
		PA_MoveBlock((void *) myblob.data(), PA_LockHandle(data), myblob.length());
		PA_UnlockHandle(data);

		return 0;
		}
	catch( Exception &error_ ) 
	{
		strlcpy(LastError, error_.what(), 255);
		return ImageMagickErrorFramework;
	}
}

int IMImage::GetProperty(long Selector, char * stringpara, double &para1, double &para2, double &para3, double &para4)
{
	if (!Initialized)
		return ImageMagickErrorUnitialized;

	if (!image.isValid())
		return ImageMagickErrorUnitialized;

	try { 
		para1 = para2 = para3 = para4 = 0;
		Geometry geo;
		std::string dummystr;

		switch (Selector)
		{
			case IM_Pref_Quality: 
				para1 = (double) image.quality();
				break;

			case IM_Pref_size: 
				para1 = image.baseColumns();
				para2 = image.baseRows();
				break;

			case IM_Pref_Geometry: 
				para1 = image.columns();
				para2 = image.rows();
				para3 = image.xResolution();
				para4 = image.yResolution();
				break;

				
			case IM_Pref_Magick:  // string
				dummystr = image.magick();
				strlcpy(stringpara, dummystr.c_str() , 255);
				break;

			case IM_Pref_Attribute:
				dummystr = image.attribute(stringpara);
				strlcpy(stringpara, dummystr.c_str() , 255);
				break;

			case IM_Pref_BackgroundColor:
				{
				Color myColor = image.backgroundColor();
				dummystr = myColor;
				strlcpy(stringpara, dummystr.c_str() , 255);
				}
				break;

			case IM_Pref_BorderColor:
				{
				Color myColor = image.borderColor();
				dummystr = myColor;
				strlcpy(stringpara, dummystr.c_str() , 255);
				}
				break;

			case IM_Pref_BoxColor:
				{
				Color myColor = image.boxColor();
				dummystr = myColor;
				strlcpy(stringpara, dummystr.c_str() , 255);
				}
				break;

			case IM_Pref_StrokeColor:
				{
				Color myColor = image.strokeColor();
				dummystr = myColor;
				strlcpy(stringpara, dummystr.c_str() , 255);
				}
				break;

			case IM_Pref_FillColor:
				{
				Color myColor = image.fillColor();
				dummystr = myColor;
				strlcpy(stringpara, dummystr.c_str() , 255);
				}
				break;

			case IM_Pref_StrokeWidth:
				para1 = image.strokeWidth();
				break;

			default: 
				return ImageMagickErrorUnknownCommand;
				break;
		}
		return 0;
		}
	catch( Exception &error_ ) 
	{
		strlcpy(LastError, error_.what(), 255);
		return ImageMagickErrorFramework;
	}
}

int IMImage::SetProperty(long Selector, char * stringpara, double para1, double para2, double para3, double para4)
{
	if (!Initialized)
		return ImageMagickErrorUnitialized;

	if (!image.isValid())
		return ImageMagickErrorUnitialized;

	try { 
		Geometry geo;

		image.modifyImage();

		switch (Selector)
		{
			case IM_Pref_Quality: 
				image.quality((const unsigned int) para1);
				break;
				
			case IM_Pref_Magick:  // string
				image.magick(stringpara);
				break;

			case IM_Pref_Attribute: // 2 x string
				{
				std::string Tag, Content, Original;
				std::string::size_type pos;
				Original = stringpara;
				stringpara[0] = 0;
				char tab = 9;
				pos = Original.find(tab, 0);
				if (pos == std::string::npos)
					return 0;
				Tag = Original.substr(0, pos);
				Content = Original.substr(pos+1, Original.length()-pos-1);
				image.attribute(Tag, Content);
				}
				break;

			case IM_Pref_BackgroundColor:
				image.backgroundColor(stringpara);
				break;

			case IM_Pref_BorderColor:
				image.borderColor(stringpara);
				break;

			case IM_Pref_BoxColor:
				image.boxColor(stringpara);
				break;

			case IM_Pref_StrokeColor:
				image.strokeColor(stringpara);
				break;

			case IM_Pref_FillColor:
				image.fillColor(stringpara);
				break;

			case IM_Pref_StrokeWidth:
				image.strokeWidth(para1);
				break;

			default: 
				return ImageMagickErrorUnknownCommand;
				break;
		}
		return 0;
		}
	catch( Exception &error_ ) 
	{
		strlcpy(LastError, error_.what(), 255);
		return ImageMagickErrorFramework;
	}
}

int IMImage::Draw(long Selector, char * stringpara, double para1, double para2, double para3, double para4, double para5, double para6)
{
	if (!Initialized)
		return ImageMagickErrorUnitialized;

	if (!image.isValid())
		return ImageMagickErrorUnitialized;

	try { 
		Geometry geo;
		image.modifyImage();

		switch (Selector)
		{
			case IM_Draw_Circle: 
				image.draw(DrawableCircle(para1, para2, para3, para4));
				break;			

			case IM_Draw_Rectangle: 
				image.draw(DrawableRectangle(para1, para2, para3, para4));
				break;			

			case IM_Draw_Arc: 
				image.draw(DrawableArc(para1, para2, para3, para4, para5, para6));
				break;			

			case IM_Draw_Color: 
				{int pm = (int) para3;
				image.draw(DrawableColor(para1, para2, (PaintMethod) pm)); }
				break;			

			case IM_Draw_Composite: 
				// "", Imageobject, x, y, width, height, composition
				// or
				// Path, 0, x, y, width, height, composition
				{
				int Composition = (int) para6;
				if (para1)
				{
					int Imageptr = (int) para1;
					IMImage * newImage;
					newImage = (IMImage *) Imageptr;	
					long CurrentPos = ValidateIMObject((long) newImage);
					if (CurrentPos >=0)
						image.draw(DrawableCompositeImage(para2, para3, para4, para5, newImage->image, (CompositeOperator) Composition));
					else
						return ImageMagickErrorUnknownObject;

				}
				else
					image.draw(DrawableCompositeImage(para2, para3, para4, para5, stringpara, (CompositeOperator) Composition));
				}
				break;			

			case IM_Draw_Ellipse: 
				image.draw(DrawableEllipse(para1, para2, para3, para4, para5, para6));
				break;			

			case IM_Draw_Line: 
				image.draw(DrawableLine(para1, para2, para3, para4));
				break;			

			case IM_Draw_Matte: 
				{	int paint = (int) para3;
					image.draw(DrawableMatte(para1, para2, (PaintMethod) paint));
				}
				break;			

			case IM_Draw_Point: 
				image.draw(DrawablePoint(para1, para2));
				break;			

			case IM_Draw_RoundRectangle: 
				image.draw(DrawableRoundRectangle(para1, para2, para3, para4, para5, para6));
				break;			

			case IM_Draw_Text: 
				{
				std::string	dummystr = "Arial";
				image.font(dummystr);
				image.draw(DrawableFont(dummystr));
				image.fontPointsize(12);
				image.penColor("black");
				image.backgroundColor("white");
				image.draw(DrawableText(20, 20, "das ist ein test"));
	
			//	image.draw(DrawableFont("Helvetica"));
			//	image.draw(DrawableGravity(NorthWestGravity));
			//	image.draw(DrawableText(para1, para2, stringpara));
				}
				break;		 

			default: 
				return ImageMagickErrorUnknownCommand;
				break;
		}
		return 0;
		}
	catch( Exception &error_ ) 
	{
		strlcpy(LastError, error_.what(), 255);
		return ImageMagickErrorFramework;
	}
}

int IMImage::ModifyImage(long Selector, char * stringpara, double para1, double para2, double para3, double para4, double para5, double para6)
{
	if (!Initialized)
		return ImageMagickErrorUnitialized;

	if (!image.isValid())
		return ImageMagickErrorUnitialized;

	try { 
		long longpara1 = 0;
		Geometry geo;
		double help1, help2;

		image.modifyImage();

		switch (Selector)
		{
			case IM_Mod_addNoise: 
				longpara1 = (long) para1;  // 0..5
				image.addNoise((const MagickLib::NoiseType) longpara1);
				break;

			case IM_Mod_flip: 
				image.flip();
				break;

			case IM_Mod_flop: 
				image.flop();
				break;

			case IM_Mod_implode: // double 0..2
				image.implode(para1);
				break;

			case IM_Mod_rotate:  // 0-360
				image.rotate(para1);
				break;

			case IM_Mod_crop:  //  x1, y1, x2, y2
				geo.xOff((long) para1);
				geo.yOff((long) para2);
				geo.width( (long) (para3-para1));
				geo.height((long) (para4-para2));
				image.crop(geo);
				break;

			case IM_Mod_blur:  // const double radius_ = 1, const double sigma_ = 0.5
				if(para2 != 0)
					image.blur(para1, para2);
				else
					image.blur(para1);
				break;

			case IM_Mod_contrast:  //sharpen
				image.contrast((long) para1);
				break;

			case IM_Mod_despeckle:  
				image.despeckle();
				break;

			case IM_Mod_adaptiveThreshold:  //  unsigned int width, unsigned int height, unsigned offset = 0
				image.adaptiveThreshold((long) para1, (long) para2, (long) para3);
				break;

			case IM_Mod_Zoom:
				geo.width((long) para1);
				geo.height((long) para2);
				image.zoom(geo);
				break;
				
			case IM_Mod_Edge:
				image.edge((long) para1);
				break;		
				
			case IM_Mod_Emboss:
				image.emboss(para1, para2);
				break;
				
			case IM_Mod_Enhance:  
				image.enhance();
				break;

			case IM_Mod_Equalize:  
				image.equalize();
				break;
				
			case IM_Mod_Frame:
				image.frame((long) para1, (long) para2, (long) para3, (long) para4);
				break;	
				
			case IM_Mod_Gamma:
				if (para1==0)
					image.gamma(para2, para3, para4);
				else
					image.gamma(para1);
				break;
				
			case IM_Mod_GaussianBlur:
				image.gaussianBlur(para1, para2);
				break;
				
			case IM_Mod_Level:
				help1 = image.quantizeColors() / 100 * para1;
				help2 = image.quantizeColors() / 100 * para2;
				image.level(help1,help2,para3);
				break;
				
			case IM_Mod_Magnify:
				image.magnify();
				break;
				
			case IM_Mod_Minify:
				image.minify();
				break;
				
			case IM_Mod_MedianFilter:
				image.medianFilter(para1);
				break;
				
			case IM_Mod_Modulate:
				image.modulate(para1, para2, para3);
				break;
				
			case IM_Mod_Negate:
				image.negate(para1 != 0);
				break;
				
			case IM_Mod_Normalize:
				image.normalize();
				break;
				
			case IM_Mod_OilPaint:
				image.oilPaint(para1);
				break;
				
			case IM_Mod_Raise:
				geo.width( (long) (para1));
				geo.height((long) (para2));
				geo.xOff((long) para3);
				geo.yOff((long) para4);				
				image.raise(geo, (para5 != 0));
				break;
				
			case IM_Mod_ReduceNoise:
				if (para1 != 0)
					image.reduceNoise(para1);
				else
					image.reduceNoise();
				break;
				
			case IM_Mod_Roll:
				image.roll((long) para1, (long) para2);
				break;
				
			case IM_Mod_Sample:
				geo.width((long) para1);
				geo.height((long) para2);				
				image.sample(geo);
				break;
				
			case IM_Mod_Scale:
				geo.width((long) para1);
				geo.height((long) para2);				
				image.scale(geo);
				break;
				
			case IM_Mod_Segment:
				image.segment(para1, para2);
				break;
				
			case IM_Mod_Shade:
				image.shade(para1, para2, (para3 != 0));
				break;
				
			case IM_Mod_Sharpen:
				image.sharpen(para1, para2);
				break;
				
			case IM_Mod_Shave:
				geo.width((long) para1);
				geo.height((long) para2);				
				image.shave(geo);
				break;
				
			case IM_Mod_Shear:
				image.shear(para1, para2);
				break;
				
			case IM_Mod_Solarize:
				image.solarize(para1);
				break;
				
			case IM_Mod_Spread:
				image.spread((int) para1);
				break;
				
			case IM_Mod_Swirl:
				image.swirl(para1);
				break;
				
			case IM_Mod_Threshold:
				image.threshold(para1);
				break;
				
			case IM_Mod_Trim:
				image.trim();
				break;
				
			case IM_Mod_Unsharpmask:
				image.unsharpmask(para1, para2, para3, para4);
				break;
				
			case IM_Mod_Wave:
				image.wave(para1, para2);
				break;
				
			default: 
				return ImageMagickErrorUnknownCommand;
				break;
		}
		return 0;
		}
	catch( Exception &error_ ) 
	{
		strlcpy(LastError, error_.what(), 255);
		return ImageMagickErrorFramework;
	}
}


char * IMImage::GetLastError(void)
{
	return LastError;
}
